/*
** (C) Copyright 2000 Gary King and Langche Zeng, All Rights Reserved.
** King@Harvard.Edu, lzeng@gwu.edu
** Current version available at http://GKing.Harvard.Edu
**
** CoxCC: Cox proportional hazard model for nested/density case control data
** (Ties handled by the Peto-Breslow method.)
**
** USAGE
**
** library maxlik;
** {b,vc,llik,ret} = coxcc(t,d,x);  @ estimate Coxcc model parameters @
** {Q,U} = coxccQ(x0,b,vc);         @ quantities of interest & uncertainty @
**
** INPUTS to CoxCC
**
** t = nx1 time indicator, where "time" is the risk set defining variable
**   (so that observations having the same value on this variable belong to
**   the same risk set.)
** d = nx1 event indicator, 1 or 0.
** x = nxk explanatory variable matrix (no constant column)
**
** GLOBALS:  
**
** _coxcc_st  = kx1 vector of starting values or
**                0 for rndn(cols(x),1)  (default)
**
** _coxcc_out = 0 if turn off unnecessary printing (default)
**              1 print intermediate results from iteration
**
**
** OUTPUTS
**
** b    = kx1 vector of estimated coefficients;
** vc   = kxk estimated covariance matrix
** llik = scalar log-likelihood at the maximum
** ret  = maxlik return code
**
**
** OUTPUT GLOBALS
**
** _coxcc_r = (sorted) risk set info, failt~nc~startr~endr~xc; 
** _coxcc_x = x (sorted by failure time)
*/

declare matrix _coxcc_st=0;
declare matrix _coxcc_out=0;

/* output globals for coxccQ.g */
declare matrix _coxcc_r=0;
declare matrix _coxcc_x=0;

proc (4)=coxcc(t,d,x);
  local sdat,k,failt,nc,startr,endr,xc,nf,nr,i,vars,lldat,b0,b,llik,g,vc,ret;

  /* check inputs */
  k=cols(x);
  if not(rows(_coxcc_st)==k or scalzero(_coxcc_st));
    "_coxcc_st error: must be 0 or a rows(X)x1 vector";end;
  endif;
    
  /* process input to extract risk sets info */
  sdat=sortc(t~d~x,1);
  t=sdat[.,1];
  d=sdat[.,2];
  x=sdat[.,3:cols(sdat)];
  failt=unique(selif(t,d.==1),1); 
  nf=rows(failt); @number of unique fails@ 
  nc=zeros(nf,1); 
  nr=zeros(nf,1); @number of total obs in each risk set@
  xc=zeros(nf,k); 
  for i(1,nf,1);
    nc[i]=sumc(selif(d,t.==failt[i]));
    nr[i]=rows(selif(d,t.==failt[i]));
    xc[i,.]=(sumc(selif(x,t.==failt[i].and d.==1)))';
  endfor;
  endr=cumsumc(nr);      
  startr=1|trimr((endr+1),0,1);

  _coxcc_r=failt~nc~startr~endr~xc; @risk sets info,sorted by failure time@
  _coxcc_x=x;                       @x values, sorted by failure time @
      
  /* feed _coxcc_r and _coxcc_x into the llik */
  let vars=_coxcc_r _coxcc_x;
  lldat=vputall(vars);
  
  /* maxlik globals */
  if scalzero(_coxcc_st);
    b0=rndn(k,1);
  else;
    b0=_coxcc_st;
  endif;
  if _coxcc_out==0;
    _max_diagnostic=miss(0,0);     @ turn off unnecessary printing @   
    __output=0;                 
  endif;
  /* _max_covpar=3;  @White se's@  don't use this. from Stata apparently 
      the "robust" thing in this context is something else. without this 
       it matches Stata */
  
  _max_gradCheckTol=0; 

  /* run maxlik */
  {b,llik,g,vc,ret}=maxlik(lldat,0,&ll,b0); 
  if scalmiss(vc);
    vc=miss(ones(k,k),1);
  endif;
  if scalmiss(b);
    b=miss(ones(k,1),1);
  endif;
  retp(b,vc,llik,ret);  
endp;

/** loglik proc **/
proc ll(b,dta);
  local x,nc,startr,endr,xc,exb,llr,i,llik; 
  call vreadall(dta);
  x=_coxcc_x;
  nc=_coxcc_r[.,2];
  startr=_coxcc_r[.,3];
  endr=_coxcc_r[.,4];
  xc=_coxcc_r[.,5:cols(_coxcc_r)];
  exb=exp(x*b);          
  llr=zeros(rows(xc),1);
  for i(1,rows(xc),1);
    llr[i]=sumc(exb[startr[i]:endr[i]]);
  endfor;
  llik=sumc(xc*b-nc.*ln(llr));
  retp(llik);
endp;  



