/*  
** (C) Copyright 2001-2005 Gary King and Langche Zeng, All Rights Reserved.
** King@Harvard.Edu, lzeng@gwu.edu
** Current version available at http://GKing.Harvard.Edu,
**   
** Distance: Compute the distance between a query point and a data matrix
**  (accept multiple query points).
**
** This procedure computes various measures on the distance between a
** query point and a data matrix, including proximity of the query point
** to the data matrix, the distance between the query point and each 
** row in the data matrix, and percentage of data within a specified
** distance of the query point. The "geometric variability", a generalized
** variance measure of the data matrix can also be computed. The distance
** measure used is based on the Gower similarity measure and is suitable
** for all types of data. Multiple query points are accepted.
**
** Usage:    D=distance(Z,X);
**
** Input:    Z = mxk matrix, the m query points.
**           X = Nxk matrix, the data matrix.
**
** Output:   D = Nxm matrix of squared distances between z in Z and rows in X,
**           or proximity of z to X (mx1), or percentage of data points 
**	     in X that are within _distance_alpha squared distance of z (mx1).
**
** Globals:  _distance_d: define output D.
**              "dist"  Distances between z and rows of X.  (default)
**              "prox"  Proximity of z to X. (Average squared distances
**                      between z and rows in X minus the generalized 
**                      variance of X.)
**              "frac"  Fraction of rows in X within _distance_alpha 
**                      square distance of z.
**        
**           _distance_alpha: the geometric variability of X, V(X), or a 
**                     scalar in (0,1). If _distance_d is "frac", 
**                      return the proportion of rows in X that are within
**                      _distance_alpha squared distance of z. Default=V(X).
**
**           _distance_v: 1: compute the geometric variability of X, V(X); 
**                        0: do not compute V(X) unless required by D. 
**                           (default)
**
** Output Global:  _distance_vx:  if _distance_v==1 or _distance_d=="prox" 
**                                or _distance_alpha=V(X): 
**                                scalar, the geometric variability of X;
**				  otherwise missing.
**
** Application (code developed for):
**  Gary King and Langche Zeng, "How Factual is Your Counterfactual?" 
**        http://GKing.Harvard.Edu
**
*/

declare matrix _distance_d="DIST";
declare matrix _distance_alpha=-1;
declare matrix _distance_v=0;
declare matrix _distance_vx={.};

proc distance(Z,X);
 local n,k,m,r,D,i,q;
      
 /* verify inputs */
  _distance_d=upper(_distance_d);
  if in(_distance_d, "DIST"|"PROX"|"FRAC",0)==0;
    "_distance_d must be DIST, PROX, or FRAC (don't forget quote marks)";end;
  endif;
  if cols(Z) /= cols(X);
    "X and Z must have the same number of columns."; end;
  endif;

  n=rows(X);
  k=cols(X);
  m=rows(Z);
  r=maxc(X)-minc(X);      @ kx1, range @    
  X=X';
  Z=Z';
  
  /* V(X) */
  if (_distance_v==1 or _distance_d$=="PROX" or _distance_alpha==-1);
     _distance_vx=zeros(n,1);  @ vector of sum of d^2 between x in X to X@
     for i(1,n,1);
       _distance_vx[i]=sumc(meanc(abs((x[.,i]-x)./r)));  
                        @sum of d^2's from x to X@
     endfor;
     _distance_vx=0.5*sumc(_distance_vx)/n^2;    @ "variance" of X @
     _distance_alpha=_distance_vx;   
  endif;
   
  /* Distance matrix */
  D=zeros(n,m);                
  for i(1,m,1);
    D[.,i]=meanc(abs((Z[.,i]-X)./r));    @squared distances @ 
  endfor;
  
  if _distance_d$=="FRAC";      @ fraction of data within specified distance@
    q=zeros(m,1);
    for i (1,m,1);
      if scalmiss(selif(D[.,i],D[.,i].<_distance_alpha));
        q[i]=0;
      else;
        q[i]=rows(selif(D[.,i],D[.,i].<_distance_alpha))/n;
      endif;
    endfor;
    D=q;
  elseif _distance_d$=="PROX";  @ Proximity @
    q=zeros(m,1);              @ vector of mean(d^2) between z to rows in X @
    for i(1,m,1);
      q[i]=meanc(D[.,i]);
    endfor;
    q=q-_distance_vx;   
    D=q;
  endif;
  
  retp(D);
endp;

























