/*  
** (C) Copyright 2001 Gary King and Langche Zeng, All Rights Reserved.
** King@Harvard.Edu, lzeng@gwu.edu
** Current version available at http://GKing.Harvard.Edu,
**   
** Overlap: Common support of propensity score distributions 
**
** This procedure returns a vector indicating which of the estimated
** propensity scores are in the overlap region of the "treatment" and
** "control" groups' propensity score distributions, and of those in the
** common support region, which (if any) are to be trimmed for having the
** lowest density. It requires dens.g.  
**
** Usage:    library pgraph;
**           #include dens.g;
**           lap=overlap(p,t);
**
** Input:    p = Nx1 vector, the estimated propensity scores
**           t = Nx1 vector, the treatment variable (t=1: treated; 0 o.w.)
**
** Output:   lap = Nx1 vector. lap=1: in common support region; lap=0: 
**           not in common support; lap=-1: in common support but to be
**           trimmed for having the lowest density.
**
** Globals:   _overlap_q: percentage of observations in common support
**             having the lowest density to be trimmed. Default is 5%.
**               (Globals of dens.g can also be set)
**
** Application (code developed for):
**  Gary King and Langche Zeng, "How Factual is Your Counterfactual?"
**         http://GKing.Harvard.Edu
**
*/
declare matrix _overlap_q=0.05;
  
proc overlap(p,t);
 local lap,id,idsort,id1,id0,do1,do0,ds1,ds0,ds,p1,p0,px,r0,r1,ntrim;
      
 /* verify inputs */
  if _overlap_q>1 or _overlap_q<0; 
    "_overlap_q must be in [0,1]";end;
  endif;
  
  id=seqa(1,1,rows(p));  /*dens.g sorts _dens_px, so need to keep track*/
  
  p1=selif(id~p,t.==1);        @ treatment group pscore @
  p1=sortc(p1,2);
  idsort=p1[.,1];
  p1=p1[.,2];
  
  p0=selif(id~p,t.==0);        @ control group pscore @
  p0=sortc(p0,2);
  idsort=idsort|p0[.,1];       @ first p1, then p0, sorted @
  p0=p0[.,2];
 
  /* density of p1(0) at p0(1) */
  _output=0; 
  _dens_px=p1;
  {px,do1}=dens(p0); @do1=0 would mean the correponding p1 is not in overlap@
  
  _dens_px=p0;
  {px,do0}=dens(p1);
  
  /* density of p1(0) at p1(0) */
  _dens_px=p1;
  {px,ds1}=dens(p1); 
  
  _dens_px=p0;
  {px,ds0}=dens(p0);
  
  /* recode and trim ds's */
  
  ds1=recode(ds1,do1.==0,-2);               @ not in common support @
      @ ("-2" and not "0", since otherwise 0's in ds1 can get confused @
  ds0=recode(ds0,do0.==0,-2);
  
  id1=seqa(1,1,rows(ds1));
  ds=sortc(ds1~id1,1);
  r0=rows(selif(ds1,ds1.==-2));             @ rows of -2's @
  r1=rows(ds1)-r0;                          @ rows in common support @
  ntrim=round(_overlap_q*r1);               @ number of rows to trim @
  if ntrim>0;
     ds[.,1]=zeros(r0,1)|(-1)*ones(ntrim,1)|ones(r1-ntrim,1);
   else;
     ds[.,1]=zeros(r0,1)|ones(r1,1);
  endif;
  ds=sortc(ds,2);
  ds1=ds[.,1];

  id0=seqa(1,1,rows(ds0));
  ds=sortc(ds0~id0,1);
  r0=rows(selif(ds0,ds0.==-2));             @ rows of -2's @
  r1=rows(ds0)-r0;                          @ rows in common support @
  ntrim=round(_overlap_q*r1);               @ number of rows to trim @
  if ntrim>0;
    ds[.,1]=zeros(r0,1)|(-1)*ones(ntrim,1)|ones(r1-ntrim,1);
  else;
    ds[.,1]=zeros(r0,1)|ones(r1,1);
  endif;
  ds=sortc(ds,2);
  ds0=ds[.,1];
        
  lap=ds1|ds0;                @ lap=0 means not in overlap, -1 means to trim @
  
  /* sort by idsort to get back the original order */
  lap=sortc(lap~idsort,2);
  lap=lap[.,1];
  retp(lap);
endp;





