/*
** (C) Copyright 1999 Gary King and Langche Zeng, All Rights Reserved.
** King@Harvard.Edu, langche@everest.fas.harvard.edu
** Current version available at http://GKing.Harvard.Edu
**
** ReLogitP: Computes Probabilities from Rare Events Logistic regression.
**
** This procedure computes probabilities from ReLogit() model coefficient
** estimates, with corrections for rare events, finite samples, and 
** selection on Y.  It implements suggestions in Gary King and 
** Langche Zeng. 1999. "Logistic Regression in Rare Events Data," 
** manuscript, available at http://GKing.Harvard.Edu.  For standard
** errors and confidence intervals on probabilities, relative risks, and
** first differences (attributable risks), see relogitQ().
**
** USAGE
**
** library maxlik;
** {b,vc,llik,ret} = relogit(y,x);  @ estimates logit parameters @
** P = relogitP(x0,b,vc);      @ computes probabilities fast @
** {Q.U} = relogitQ(x1,b,vc);  @ probabilities, first differences, relative 
**              risks, along with standard errors or confidence intervals @
**
** INPUTS to relogitP
**
** x0 = qxk matrix of values of explanatory variables, with constant;
**      each row produces one probability.
** b  = kx1 vector of estimated logit coefficients
** vc = kxk estimated variance matrix of b
**      (b & vc are any outputs from relogit, preferably using 
**      bias correction and, if necessary, weighting or prior correction)
**
** OUTPUT
**
** p = Pr(Y=1|X), an qx1 vector
**
** GLOBAL
**
** _relogitp_c = 0  Pr(Y=1|X,b) instead (the usual plug-in approach; not
**                   generally recommended)
**               1   approximate Bayes with analytical approximation (default)
**               2|m approximate Bayes with m multivariate normal simulations
**               3   approximate unbiased estimator
**
*/
declare matrix _relogitp_c=1;
proc relogitp(x,b,vc);
  local p,k,bsims,rx;

  if _relogitp_c[1]==0;             @ Pr(Y=1|betahat) @
    p=logiti(x*b);
  
  elseif _relogitp_c[1]==1;         @ Pr(Y=1) analytically @
    p=logiti(x*b);
    p=p+(p-p^2).*(0.5-p).*sumc((x.*(x*vc))');
  
  elseif _relogitp_c[1]==2;         @ Pr(Y=1) computed by normal simulation @
    rx=rows(x);
    bsims=rndmn(b,vc,_relogitp_c[2])';
    if (rx*_relogitp_c[2])<1000000;	@ to save time @
      p=meanc(logiti(x*bsims)');
    else;				@ to save memory  @
      p={};
      for k(1,rx,1);
        p=p|meanc(logiti(x[k,.]*bsims)');
      endfor;
    endif;
    
  elseif _relogitp_c[1]==3;         @ bias correction @
    p=logiti(x*b);
    p=p-(p-p^2).*(0.5-p).*sumc((x.*(x*vc))');
  
  else;
    "_relogitp_c is wrong";
    end;
    
  endif;
  
  p=recode(p,(p.<0)~(p.>1),0|1);
  retp(p);
endp;
